@@ -1,6 +1,7 @@
 #define BITS_TEST_C
-#include "../../src/bits.h"
-#include "../../src/common.h"
+#include "config.h"
+#include "bits.h"
+#include "common.h"
 #include <string.h>
 #include <stdlib.h>
 #include "tests_common.h"
@@ -327,6 +328,7 @@ main (int argc, char const *argv[])
   unsigned char sentinel[]
       = { 0xCF, 0x7B, 0x1F, 0x23, 0xFD, 0xDE, 0x38, 0xA9,
           0x5F, 0x7C, 0x68, 0xB8, 0x4E, 0x6D, 0x33, 0x5F };
+  loglevel = is_make_silent() ? 0 : 3;
 
   bit_read_B_tests ();
   bit_write_B_tests ();
@@ -589,24 +591,70 @@ main (int argc, char const *argv[])
   else
     fail ("bit_calc_CRC %X", bs);
   bit_advance_position (&bitchain, 16L);
+  pos = bit_position (&bitchain);
 
-  bit_write_TV (&bitchain, (char *)"GNU"); // now we store the \0 also
+  bit_write_TV (&bitchain, (char *)"GNU"); // we don't store the \0
   if (bitchain.byte == 70 && bitchain.bit == 2)
     pass ();
   else
     fail ("bit_write_TV @%ld.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -42L);
+  bit_set_position (&bitchain, pos);
   str = bit_read_TV (&bitchain);
   if (!strcmp (str, "GNU"))
     pass ();
   else
     fail ("bit_read_TV");
+  bit_set_position (&bitchain, pos);
   free (str);
-  bit_advance_position (&bitchain, -8L);
+  for (int i = 0; i < 6; i++)
+    bit_write_B (&bitchain, 0); // padding for the T BS, to have aligned strings at 67
+  pos = bit_position (&bitchain); // 526
+
+  {
+    BITCODE_T wstr;
+    const uint16_t exp[] = {'T', 'e', 'i', 'g', 'h', 'a', 0x2122, 0};
+    bitchain.version = R_2007; // @65.6
+    bit_write_T (&bitchain, (char *)"Teigha\\U+2122"); // convert to unicode
+    if (bitchain.byte == 83 && bitchain.bit == 0 &&    // containing the ending 0L
+        bitchain.chain[79] == 0x22 &&
+        bitchain.chain[80] == 0x21)
+      pass ();
+    else
+      fail ("bit_write_T => TU @%ld.%d", bitchain.byte, bitchain.bit);
+
+    bit_set_position (&bitchain, pos);
+    bitchain.from_version = R_2007;
+    bitchain.version = R_2000;
+    wstr = bit_read_T (&bitchain);
+    if (wstr && !memcmp (wstr, exp, sizeof (exp)))
+      pass ();
+    else
+      fail ("bit_read_T => TU @%ld.%d", bitchain.byte, bitchain.bit);
+    bit_set_position (&bitchain, pos);
+    free (wstr);
+
+    bit_write_T (&bitchain, (char*)exp); // convert to ASCII via embed
+    if (bitchain.byte == 81 && bitchain.bit == 0)
+      pass ();
+    else
+      fail ("bit_write_T => TV \"%s\" @%ld.%d", &bitchain.chain[67],
+            bitchain.byte, bitchain.bit);
+
+    bit_set_position (&bitchain, pos);
+    bitchain.version = R_2000;
+    bitchain.from_version = R_2007;
+    str = bit_read_T (&bitchain);
+    if (str && !strcmp (str, "Teigha\\U+2122"))
+      pass ();
+    else
+      fail ("bit_read_T => TV \"%s\" @%ld.%d", str, bitchain.byte, bitchain.bit);
+    bitchain.from_version = bitchain.version = R_2004;
+    free (str);
+  }
 
   bit_write_L (&bitchain, 20);
-  if (bitchain.byte == 73 && bitchain.bit == 2)
+  if (bitchain.byte == 99 && bitchain.bit == 0)
     pass ();
   else
     fail ("bit_write_L @%ld.%d", bitchain.byte, bitchain.bit);
@@ -620,14 +668,11 @@ main (int argc, char const *argv[])
   pos = bit_position (&bitchain);
   {
     Dwg_Color color;
-    bitchain.version = R_2000;
+    bitchain.from_version = bitchain.version = R_2000;
+    memset (&color, 0, sizeof (color));
     color.index = 19;
-    color.rgb = 5190965;
-    color.flag = 1;
-    color.name = (char *)"Some name";
-    color.book_name = (char *)"book_name";
-    bit_write_CMC (&bitchain, &color);
-    if (bitchain.byte == 74 && bitchain.bit == 4)
+    bit_write_CMC (&bitchain, &bitchain, &color);
+    if (bitchain.byte == 100 && bitchain.bit == 2)
       pass ();
     else
       fail ("bit_write_CMC @%ld.%d", bitchain.byte, bitchain.bit);
@@ -641,39 +686,45 @@ main (int argc, char const *argv[])
   bit_advance_position (&bitchain, -size);
   {
     Dwg_Color color_read;
-    bit_read_CMC (&bitchain, &color_read);
+    bit_read_CMC (&bitchain, &bitchain, &color_read);
 
     if (color_read.index == 19)
       pass ();
     else
-      fail ("bit_read_CMC %d", color_read.index);
+      fail ("bit_read_CMC %d (r2000)", color_read.index);
+    free (color_read.name);
+    free (color_read.book_name);
   }
 
   bit_advance_position (&bitchain, -size);
   {
     Dwg_Color color;
-    bitchain.version = R_2004;
+    pos = bit_position (&bitchain);
+    bitchain.from_version = bitchain.version = R_2004;
+    memset (&color, 0, sizeof (color));
     color.index = 19;
-    color.rgb = 5190965;
-    color.flag = 1;
-    color.name = (char *)"Some name";
-    color.book_name = (char *)"book_name";
-    bit_write_CMC (&bitchain, &color);
-    if (bitchain.byte == 91 && bitchain.bit == 0)
+    color.rgb = 0xc2ff00ff;
+    color.flag = 3; // write both. to skip book set it to NULL
+    color.name = (char *)"Some color";
+    color.book_name = (char *)"DIC(3) Catalog";
+    bit_write_CMC (&bitchain, &bitchain, &color);
+    if (bitchain.byte == 131 && bitchain.bit == 0)
       pass ();
     else
-      fail ("bit_write_CMC @%ld.%d", bitchain.byte, bitchain.bit);
+      fail ("bit_write_CMC (r2004) @%ld.%d", bitchain.byte, bitchain.bit);
 
     bit_set_position (&bitchain, pos);
-    bit_read_CMC (&bitchain, &color);
+    bit_read_CMC (&bitchain, &bitchain, &color);
 
-    if (color.index == 19 && color.rgb == 5190965 && color.flag == 1
-        && !strcmp (color.name, "Some name"))
+    // index is fixed up for dxf
+    if (color.rgb == 0xc2ff00ff && color.method == 0xc2
+        && color.flag == 3 && !strcmp (color.name, "Some color"))
       pass ();
     else
-      fail ("bit_read_CMC (%d,%u,%d) %s", color.index, color.rgb, color.flag,
+      fail ("bit_read_CMC (%d,%x,%d) %s", color.index, color.rgb, color.flag,
             color.name);
     free (color.name);
+    free (color.book_name);
   }
 
   bitchain.byte++;
@@ -691,22 +742,25 @@ main (int argc, char const *argv[])
   bitchain.byte = 0;
   {
     int ret = bit_search_sentinel (&bitchain, sentinel);
-    if (bitchain.byte == 108)
+    if (bitchain.byte == 148)
       pass ();
     else
-      fail ("bit_search_sentinel %lu", bitchain.byte);
+      {
+        fail ("bit_search_sentinel %lu", bitchain.byte);
+        bitchain.byte = 150;
+      }
   }
   {
     unsigned int check
-        = bit_calc_CRC (0xC0C1, (unsigned char *)bitchain.chain, 108L);
-    if (check == 0xEB57)
+        = bit_calc_CRC (0xC0C1, (unsigned char *)bitchain.chain, 124L);
+    if (check == 0x43B8)
       pass ();
     else
       fail ("bit_calc_CRC %04X", check);
   }
 
   bit_chain_alloc (&bitchain);
-  if (bitchain.size == 82020)
+  if (bitchain.size == 8292)
     pass ();
   else
     fail ("bit_chain_alloc %ld", bitchain.size);
@@ -731,5 +785,5 @@ main (int argc, char const *argv[])
   }
 
   free (bitchain.chain);
-  return 0;
+  return failed;
 }
