@@ -6,10 +6,11 @@ void
 api_process (dwg_object *obj)
 {
   int error;
-  BITCODE_BS class_version; /*!< r2010+ =3 */
+  ANNOTSCALEOBJECTCONTEXTDATA_fields;
+  /*BITCODE_BS class_version; // r2010+ =3
   BITCODE_B is_default;
   BITCODE_B has_xdic;
-  BITCODE_H scale; /* DXF 340 */
+  BITCODE_H scale; */
 
   Dwg_Version_Type dwg_version = obj->parent->header.version;
 #ifdef DEBUG_CLASSES
@@ -17,7 +18,7 @@ api_process (dwg_object *obj)
 
   CHK_ENTITY_TYPE (_obj, MLEADEROBJECTCONTEXTDATA, class_version, BS);
   CHK_ENTITY_TYPE (_obj, MLEADEROBJECTCONTEXTDATA, is_default, B);
-  CHK_ENTITY_TYPE (_obj, MLEADEROBJECTCONTEXTDATA, has_xdic, B);
   CHK_ENTITY_H (_obj, MLEADEROBJECTCONTEXTDATA, scale);
+  // ...
 #endif
 }
